
use adventureworksdw
go

select count(*) from factinternetsales
go

-- turn on Show Actual Execution Plan 

-- rollup = union all of n groupings
-- cube = union all of 2 to the nth power groupings

-- one group per PK, 158 rows
select productkey, sum(extendedamount)
from factinternetsales
group by productkey

-- subtotals by productkey
-- one dimension, rollup adds total n+1 = 2
select productkey, sum(extendedamount)
from factinternetsales
group by productkey with rollup

-- eqivalent with union all
-- but has to scan table twice
select productkey, sum(extendedamount)
from factinternetsales
group by productkey
union all
select null, sum(extendedamount)
from factinternetsales

-- one dimension, cube == rollup 2 to the first power = 2
select productkey, sum(extendedamount)
from factinternetsales
group by productkey with cube

-- group for each productkey, STkey combo, 1149 rows
select productkey, salesterritorykey, sum(extendedamount)
from factinternetsales
group by productkey, salesterritorykey 

-- one subtotal per product, one grand total, 
-- one group by product, territory
-- two sets of rollup totals n+1 == 2+1 == 3
select productkey, salesterritorykey, sum(extendedamount)
from factinternetsales
group by productkey, salesterritorykey with rollup

-- one set for each salesterritory, one set for each product, 
-- grand total, group by product, sales = 4 groups
select productkey, salesterritorykey, sum(extendedamount)
from factinternetsales
group by productkey, salesterritorykey with cube
order by productkey, salesterritorykey asc

-- 59051 rows - one for each product key, salesterritory, customer
select productkey, salesterritorykey, customerkey, sum(extendedamount)
from factinternetsales
group by productkey, salesterritorykey, customerkey

-- grand total
-- total by productkey
-- total by productkey, salesterritory
-- by productkey, salesterritory, customer
-- 3 sets of total + 1 = 4 groupings
select productkey, salesterritorykey, customerkey, sum(extendedamount)
from factinternetsales
group by productkey, salesterritorykey, customerkey with rollup
order by productkey, salesterritorykey, customerkey

-- grand total
-- by customer
-- by salesterr
-- by salesterr, customer
-- by prod
-- by prod, customer
-- by prod, salesterr
-- by prod, salesterr, customer
-- 8 groupings = 2 to the third power
select productkey, salesterritorykey, customerkey, sum(extendedamount)
from factinternetsales
-- cut down the sheer numbers in the raw data
where salesterritorykey < 5 
  and productkey < 220 
  and customerkey < 11500
group by productkey, salesterritorykey, customerkey with cube
order by productkey, salesterritorykey, customerkey

-- suppose I only want
-- grand total
-- by prod, salesterr, customer
-- by prod, salesterr
select productkey, salesterritorykey, customerkey, sum(extendedamount)
from factinternetsales
-- cut down the sheer numbers in the raw data
where salesterritorykey < 5 
  and productkey < 220 
  and customerkey < 11500
group by grouping sets ((productkey, salesterritorykey, customerkey), (productkey, salesterritorykey), ())
go