
use tempdb
go

--
-- Step 1: Simple example using INSERT AND UPDATE statements
--

drop table target
drop table staging 

create table target (id int, name varchar(20), age int)
go
insert target values(1, 'Bob', 52)
insert target values(2, 'Fred', 33)
insert target values(3, 'Joe', 34)
go

create table staging (id int, name varchar(20), age int)
insert staging values(1, 'Bob', 53)
insert staging values(2, 'Jim', 87)
insert staging values(5, 'Janet', 45)
go

begin transaction;
-- statement 1
insert target 
select * from staging
where id not in (select id from target);
-- statement 2
update t
set name = s.name,
    age  = s.age 
from target t 
join staging s on (t.id = s.id);
commit;

select * from target
go

--
-- Step 2: Simple example using INSERT AND UPDATE statements
--         This can produce anamolies when multiple staging rows for same target row.
--

drop table target
drop table staging 

create table target (id int, name varchar(20), age int)
go
insert target values(1, 'Bob', 52)
insert target values(2, 'Fred', 33)
insert target values(3, 'Joe', 34)
go

create table staging (id int, name varchar(20), age int)
insert staging values(1, 'Bob', 53)   -- two rows for ID 1
insert staging values(1, 'Bill', 55)  -- two rows for ID 1
insert staging values(2, 'Jim', 87)
insert staging values(5, 'Janet', 45)
go

-- two rows with an ID of 1 qualify
begin transaction;
-- statemeent 1
insert target 
select * from staging
where id not in (select id from target);
-- statement 2 
update t
set t.name = s.name,
    t.age  = s.age 
from target t 
join staging s on (t.id = s.id);
commit;
go

-- updated using one row, one row lost
select * from target;
go

--
-- Step 3: Simple example using MERGE
--         This cannot produce anamolies when multiple staging rows for same target row.
--         Error is produced instead
--

drop table target
drop table staging 

create table target (id int, name varchar(20), age int)
go
insert target values(1, 'Bob', 52)
insert target values(2, 'Fred', 33)
insert target values(3, 'Joe', 34)
go

create table staging (id int, name varchar(20), age int)
insert staging values(1, 'Bob', 53)   -- two rows for ID 1
insert staging values(1, 'Bill', 55)  -- two rows for ID 1
insert staging values(2, 'Jim', 87)
insert staging values(5, 'Janet', 45)
go

--
-- Msg 8672, Level 16, State 1, Line 1
-- The MERGE statement attempted to UPDATE or DELETE the same row more than once. This happens when a target row matches more than one source row. A MERGE statement cannot UPDATE/DELETE the same row of the target table multiple times. Refine the ON clause to  ensure a target row matches at most one source row, or use the GROUP BY clause to group the source rows.
merge [target] t
  using staging s
  on t.id = s.id
  when matched then
    update set t.name = s.name, t.age = s.age;
go
 