﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.IO;
using System.Runtime.InteropServices;
using Microsoft.Win32.SafeHandles;

namespace FilestreamReader
{
    class Program
    {
        [DllImport("sqlncli10.dll", SetLastError = true, CharSet = CharSet.Unicode)]
        public static extern SafeFileHandle OpenSqlFilestream(
                    string FilestreamPath,
                    UInt32 DesiredAccess,
                    UInt32 OpenOptions,
                    byte[] FilestreamTransactionContext,
                    UInt32 FilestreamTransactionContextLength,
                    LARGE_INTEGER_SQL AllocationSize);

        [StructLayout(LayoutKind.Sequential)]
        public struct LARGE_INTEGER_SQL
        {
            public Int64 QuadPart;
            public LARGE_INTEGER_SQL(Int64 quadPart) { QuadPart = quadPart; }
        }

        //Get Last Error to check if the OpenSqlFilestream succeeded in returning a valid / handle
        //
        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern UInt32 GetLastError();

        // DesiredAccess
        //
        const UInt32 DESIRED_ACCESS_READ = 0x00000000;
        const UInt32 DESIRED_ACCESS_WRITE = 0x00000001;
        const UInt32 DESIRED_ACCESS_READWRITE = 0x00000002;

        static void Main(string[] args)
        {
            string connectionString = "server=.;integrated security=sspi;database=filestreamdb";

            string filePath = string.Empty;
            SqlConnection sqlConnection = new SqlConnection(connectionString);
            sqlConnection.Open();

            SqlTransaction tx = sqlConnection.BeginTransaction();

            SqlCommand cmd = new SqlCommand();
            cmd.CommandText = "SELECT TOP(1) Resume.PathName() FROM dbo.student where name = 'Fred'";
            cmd.CommandType = CommandType.Text;
            cmd.Connection = sqlConnection;
            cmd.Transaction = tx;

            Object pathObj = cmd.ExecuteScalar();
            if (DBNull.Value != pathObj)
            {
                filePath = (string)pathObj;
            }

            byte[] transactionToken;

            cmd.CommandText = "SELECT GET_FILESTREAM_TRANSACTION_CONTEXT()";
            cmd.CommandType = CommandType.Text;
            cmd.Connection = sqlConnection;

            Object txObj = cmd.ExecuteScalar();
            if (DBNull.Value != txObj)
            {
                transactionToken = (byte[])txObj;
            }
            else
            {
                transactionToken = null;
                // bail, an error
            }

            // Read a file

            SafeFileHandle handle = OpenSqlFilestream(
                         filePath,
                         DESIRED_ACCESS_READ,
                         0,
                         transactionToken,
                         (UInt32)transactionToken.Length,
                         new LARGE_INTEGER_SQL(0));

            UInt32 NtStatus = GetLastError();


            int bufferSize = 4096;
            byte[] buffer = new byte[4096];

            // The following code demonstrates how to read the file
            FileStream fs = new FileStream(handle,
                                           FileAccess.Read,
                                           bufferSize,
                                           false);

            byte[] readBytes = new byte[buffer.Length];
            int numBytesRead = fs.Read(readBytes, 0, buffer.Length);

            if (0 != numBytesRead)
            {
                Console.WriteLine("bytes read: {0}", BitConverter.ToString(readBytes));
                // PrintContent(readBytes);
            }

            fs.Close();

            if (handle != null && !handle.IsClosed)
            {
                Console.WriteLine("\n\nClosing handle");
                handle.Close();
            }

            tx.Commit();
        }
    }
}
